@@ -79,17 +79,19 @@ impl TryFrom<(&[u8], u16)> for ExtendedCapabilities {
             let i9 = input[9];
 
             if i8 > 1 {
-                return Err(Error::ParseError(
-                    format!("Illegal value '{}' for pin_block_2_format_support", i8).into(),
-                ));
+                return Err(Error::ParseError(format!(
+                    "Illegal value '{}' for pin_block_2_format_support",
+                    i8
+                )));
             }
 
             pin_block_2_format_support = Some(i8 != 0);
 
             if i9 > 1 {
-                return Err(Error::ParseError(
-                    format!("Illegal value '{}' for mse_command_support", i9).into(),
-                ));
+                return Err(Error::ParseError(format!(
+                    "Illegal value '{}' for mse_command_support",
+                    i9
+                )));
             }
             mse_command_support = Some(i9 != 0);
         }
