@@ -125,6 +125,8 @@ pub(crate) fn generate_asymmetric_key_pair(
     card_tx: &mut OpenPgpTransaction,
     key_type: KeyType,
 ) -> Result<Tlv, Error> {
+    log::info!("OpenPgpTransaction: generate_asymmetric_key_pair");
+
     // generate key
     let crt = control_reference_template(key_type)?;
     let gen_key_cmd = commands::gen_key(crt.serialize().to_vec());
@@ -147,6 +149,8 @@ pub(crate) fn public_key(
     card_tx: &mut OpenPgpTransaction,
     key_type: KeyType,
 ) -> Result<PublicKeyMaterial, Error> {
+    log::info!("OpenPgpTransaction: public_key");
+
     // get current algo
     let ard = card_tx.application_related_data()?; // FIXME: caching
     let algo = ard.algorithm_attributes(key_type)?;
@@ -175,6 +179,8 @@ pub(crate) fn key_import(
     key_type: KeyType,
     algo_info: Option<AlgoInfo>,
 ) -> Result<(), Error> {
+    log::info!("OpenPgpTransaction: key_import");
+
     // FIXME: caching?
     let ard = card_tx.application_related_data()?;
 
@@ -316,8 +322,7 @@ fn card_algo_rsa(algo_info: AlgoInfo, key_type: KeyType, rsa_bits: u16) -> Resul
     // Get RSA algo attributes
     let rsa_algos: Vec<_> = keytype_algos
         .iter()
-        .map(|a| if let Algo::Rsa(r) = a { Some(r) } else { None })
-        .flatten()
+        .filter_map(|a| if let Algo::Rsa(r) = a { Some(r) } else { None })
         .collect();
 
     // Filter card algorithms by rsa bitlength of the key we want to upload
@@ -334,9 +339,10 @@ fn card_algo_rsa(algo_info: AlgoInfo, key_type: KeyType, rsa_bits: u16) -> Resul
         Ok((**algo.last().unwrap()).clone())
     } else {
         // RSA with this bit length is not in algo_info
-        return Err(Error::UnsupportedAlgo(
-            format!("RSA {} unsupported according to algo_info", rsa_bits).into(),
-        ));
+        return Err(Error::UnsupportedAlgo(format!(
+            "RSA {} unsupported according to algo_info",
+            rsa_bits
+        )));
     }
 }
 
@@ -350,8 +356,7 @@ fn check_card_algo_ecc(algo_info: AlgoInfo, key_type: KeyType, oid: &[u8]) -> Ve
     // Get attributes
     let ecc_algos: Vec<_> = keytype_algos
         .iter()
-        .map(|a| if let Algo::Ecc(e) = a { Some(e) } else { None })
-        .flatten()
+        .filter_map(|a| if let Algo::Ecc(e) = a { Some(e) } else { None })
         .collect();
 
     // Find entries with this OID in the algorithm information for key_type
